import {Component} from '@angular/core';
import {Location} from '@angular/common';
import {FormBuilder, FormGroup, ReactiveFormsModule, Validators} from "@angular/forms";
import {NavigationService} from "@service/navigation.service";
import {passwordMatchValidator} from '@service/password-match.validator';
import SetupService from "@api/v1/setup.service";
import {MatFormFieldModule} from "@angular/material/form-field";
import {MatSelectModule} from "@angular/material/select";
import {MatInput} from "@angular/material/input";
import {MatTooltip} from "@angular/material/tooltip";
import {MatIconButton} from "@angular/material/button";
import {MatIcon} from "@angular/material/icon";
import AuthApiService from "@api/v1/authApiService";
import {OAuthProviders} from "@component/OAuthProviders/OAuthProviders";
import {OAuthProvider} from "@model/oAuthProvider";
import {PasswordInputComponent} from "@component/common/password-input/password-input.component";
import {OAuthAction} from "@model/oAuthAction";
import {Router} from "@angular/router";

@Component({
  selector: 'app-create-admin',
  standalone: true,
  imports: [
    ReactiveFormsModule,
    MatFormFieldModule,
    MatSelectModule,
    MatInput,
    MatTooltip,
    MatIconButton,
    MatIcon,
    OAuthProviders,
    PasswordInputComponent
  ],
  templateUrl: './create-admin.component.html',
  providers: [AuthApiService, Location]
})

export class CreateAdminComponent {
  protected createAdminForm!: FormGroup;
  protected hideRetypePass = true;
  protected activatedProviders: OAuthProvider[] = [];

  constructor(private router: Router,
              private location: Location,
              private navigationService: NavigationService, private formBuilder: FormBuilder, private api: SetupService) {
    this.createAdminForm = this.formBuilder.group({
        user: ['', Validators.pattern(/^([A-Za-z0-9]){4,}$/)],
        email: ['', Validators.email],
        password: ['', Validators.required],
        retype: ['', Validators.required]
      },
      {validators: passwordMatchValidator('password', 'retype')}
    );

    this.navigationService.setNextButtonState(false);
    this.createAdminForm.valueChanges.subscribe(() => {
      this.navigationService.setNextButtonState(this.createAdminForm.valid);
    });

    this.navigationService.nextButtonAction = () => {
      return this.api.createAdmin({
          "email": this.createAdminForm.get('email')?.value,
          "username": this.createAdminForm.get('user')?.value,
          "password": this.createAdminForm.get('password')?.value
        }
      );
    };

    this.updateAdminData();
  }

  private updateAdminData() {
    this.api.adminConfiguration().subscribe(configuration => {
      if (configuration) {
        if (this.createAdminForm.get('email')?.value == 0)
          this.createAdminForm.get('email')?.setValue(configuration.email);

        if (this.createAdminForm.get('user')?.value == 0)
          this.createAdminForm.get('user')?.setValue(configuration.username);

        this.activatedProviders = configuration.usedOAuthProviders;
      }

      const currentPath = this.router.url.split('?')[0];
      this.location.replaceState(currentPath);
    });
  }

  protected toggleRetypePassword(event: MouseEvent) {
    this.hideRetypePass = !this.hideRetypePass;
    event.stopPropagation();
  }

  protected updateProviders() {
    this.updateAdminData();
  }

  protected readonly OAuthAction = OAuthAction;
}
